@@ -67,7 +67,7 @@ size_t cacheSize_RecentUrl(const iRecentUrl *d) {
         size += size_String(&d->cachedResponse->meta);
         size += size_Block(&d->cachedResponse->body);
     }
-    return size;    
+    return size;
 }
 
 size_t memorySize_RecentUrl(const iRecentUrl *d) {
@@ -177,6 +177,10 @@ iString *debugInfo_History(const iHistory *d) {
 }
 
 void serialize_History(const iHistory *d, iStream *outs) {
+    serializeWithContent_History(d, outs, iTrue);
+}
+
+void serializeWithContent_History(const iHistory *d, iStream *outs, iBool withContent) {
     lock_Mutex(d->mtx);
     writeU16_Stream(outs, d->recentPos);
     writeU16_Stream(outs, size_Array(&d->recent));
@@ -185,7 +189,7 @@ void serialize_History(const iHistory *d, iStream *outs) {
         serialize_String(&item->url, outs);
         write32_Stream(outs, item->normScrollY * 1.0e6f);
         writeU16_Stream(outs, item->flags);
-        if (item->cachedResponse) {
+        if (withContent && item->cachedResponse) {
             write8_Stream(outs, 1);
             serialize_GmResponse(item->cachedResponse, outs);
         }
@@ -323,7 +327,7 @@ void undo_History(iHistory *d) {
         deinit_RecentUrl(back_Array(&d->recent));
         popBack_Array(&d->recent);
     }
-    unlock_Mutex(d->mtx);    
+    unlock_Mutex(d->mtx);
 }
 
 iRecentUrl *precedingLocked_History(iHistory *d) {
@@ -421,7 +425,7 @@ void setIdentity_History(iHistory *d, const iBlock *identityFingerprint) {
             clear_Block(&item->setIdentity);
         }
     }
-    unlock_Mutex(d->mtx);    
+    unlock_Mutex(d->mtx);
 }
 
 void setCachedResponse_History(iHistory *d, const iGmResponse *response) {
@@ -438,9 +442,11 @@ void setCachedResponse_History(iHistory *d, const iGmResponse *response) {
 }
 
 void setCachedDocument_History(iHistory *d, iGmDocument *doc) {
+    if (size_GmDocument(doc).x == 0) {
+        return;
+    }
     lock_Mutex(d->mtx);
     iRecentUrl *item = mostRecentUrl_History(d);
-    iAssert(size_GmDocument(doc).x > 0);
     if (item) {
 #if !defined (NDEBUG)
         if (!equal_String(url_GmDocument(doc), &item->url)) {
